@@ -96,6 +96,13 @@ layer_surface_configure_xdg_surface (LayerSurface *self, uint32_t serial, gboole
         height = self->last_layer_configured_size.height;
     }
 
+    if (!self->has_initial_layer_shell_configure) {
+        // skip sending xdg_toplevel and xdg_surface configure to GTK
+        // since layer shell surface has not been configured yet and
+        // attaching a buffer to an unconfigured surface is a protocol error
+        return;
+    }
+
     if (self->cached_xdg_configure_size.width != width ||
         self->cached_xdg_configure_size.height != height ||
         send_even_if_size_unchanged
@@ -152,6 +159,7 @@ layer_surface_handle_configure (void *data,
     (void)surface;
     LayerSurface *self = data;
     self->last_layer_configured_size = (GtkRequisition){w, h};
+    self->has_initial_layer_shell_configure = true;
     layer_surface_configure_xdg_surface (self, serial, TRUE);
 }
 
@@ -205,6 +213,7 @@ layer_surface_create_surface_object (LayerSurface *self, struct wl_surface *wl_s
     }
 
     enum zwlr_layer_shell_v1_layer layer = gtk_layer_shell_layer_get_zwlr_layer_shell_v1_layer(self->layer);
+    self->has_initial_layer_shell_configure = false;
     self->layer_surface = zwlr_layer_shell_v1_get_layer_surface (layer_shell_global,
                                                                  wl_surface,
                                                                  output,
@@ -235,6 +244,7 @@ layer_surface_unmap (LayerSurface *super)
 
     self->client_facing_xdg_surface = NULL;
     self->client_facing_xdg_toplevel = NULL;
+    self->has_initial_layer_shell_configure = false;
 
     self->cached_xdg_configure_size = (GtkRequisition){-1, -1};
     self->cached_layer_size_set = (GtkRequisition){-1, -1};
@@ -324,6 +334,7 @@ layer_surface_new (GtkWindow *gtk_window)
     LayerSurface *self = g_new0 (LayerSurface, 1);
     self->cached_xdg_configure_size = (GtkRequisition){-1, -1};
     self->cached_layer_size_set = (GtkRequisition){-1, -1};
+    self->has_initial_layer_shell_configure = false;
     all_layer_surfaces = g_list_append (all_layer_surfaces, self);
 
     self->gtk_window = gtk_window;
@@ -538,15 +549,18 @@ gint find_layer_surface_with_wl_surface (gconstpointer layer_surface, gconstpoin
     return wl_surface == needle ? 0 : 1;
 }
 
-struct wl_proxy *
+gboolean
 layer_surface_handle_request (
     struct wl_proxy *proxy,
     uint32_t opcode,
     const struct wl_interface *interface,
     uint32_t version,
     uint32_t flags,
-    union wl_argument *args)
+    union wl_argument *args,
+    struct wl_proxy **ret_proxy)
 {
+    (void)interface;
+    (void)flags;
     const char* type = proxy->object.interface->name;
     if (strcmp(type, xdg_wm_base_interface.name) == 0) {
         if (opcode == XDG_WM_BASE_GET_XDG_SURFACE) {
@@ -563,7 +577,8 @@ layer_surface_handle_request (
                     self);
                 self->client_facing_xdg_surface = (struct xdg_surface *)xdg_surface;
                 layer_surface_create_surface_object(self, wl_surface);
-                return xdg_surface;
+                *ret_proxy = xdg_surface;
+                return TRUE;
             }
         }
     } else if (strcmp(type, xdg_surface_interface.name) == 0) {
@@ -576,13 +591,15 @@ layer_surface_handle_request (
                         NULL,
                         (struct xdg_positioner *)args[2].o);
                     zwlr_layer_surface_v1_get_popup (self->layer_surface, xdg_popup);
-                    return (struct wl_proxy *)xdg_popup;
+                    *ret_proxy = (struct wl_proxy *)xdg_popup;
+                    return TRUE;
                 } else {
                     g_critical ("tried to create popup before layer shell surface");
-                    return libwayland_shim_create_client_proxy (proxy, &xdg_popup_interface, version, NULL, NULL, NULL);
+                    *ret_proxy = libwayland_shim_create_client_proxy (proxy, &xdg_popup_interface, version, NULL, NULL, NULL);
+                    return TRUE;
                 }
             }
         }
     }
-    return libwayland_shim_real_wl_proxy_marshal_array_flags (proxy, opcode, interface, version, flags, args);
+    return FALSE;
 }
